#include "cluster_reader.h"
#include <assert.h>

int read_clusters(FILE *fp, int64_t n_clusters, Cluster *buf, int *n_left) {
    int iframe = 0;
    int nph = *n_left;

    size_t nph_read = 0;
    size_t nn = *n_left;
    size_t nr = 0;

    // read photons left from previous frame
    if (nph) {
        if (nph > n_clusters) {
            // if we have more photons left in the frame then photons to read we
            // read directly
            nn = n_clusters;
        } else {
            nn = nph;
        }
        nr += fread((void *)(buf + nph_read), sizeof(Cluster), nn, fp);
        nph_read += nn;
        *n_left = nph - nn;
    }
    if (nph_read < n_clusters) {
        // keep on reading frames and photons until reaching n_clusters
        while (fread(&iframe, sizeof(iframe), 1, fp)) {

	  
            if (fread(&nph, sizeof(nph), 1, fp)) {
                if (nph > n_clusters - nph_read)
                    nn = n_clusters - nph_read;
                else
                    nn = nph;

                nr += fread((void *)(buf + nph_read), sizeof(Cluster), nn, fp);
                nph_read += nn;
                *n_left = nph - nn;
            }
            if (nph_read >= n_clusters)
                break;



        }
    }
    assert(nph_read <= n_clusters); // sanity check in debug mode
    return nph_read;
}



int read_clusters_with_cut(FILE *fp, int64_t n_clusters, Cluster *buf, int *n_left, double *noise_map, int nx, int ny) {
    int iframe = 0;
    int nph = *n_left;

    size_t nph_read = 0;

    int32_t tot2[4], t2max, tot1;
    int32_t val, tot3;
    Cluster *ptr=buf;
    int good=1;
    double noise;
    // read photons left from previous frame
    if (nph) {
	for (int iph=0; iph<nph; iph++) {
	  // read photons 1 by 1
	  fread((void *)(ptr), sizeof(Cluster), 1, fp);
	  good=1;
	  if (noise_map) {
	    if (ptr->x>=0 && ptr->x<nx &&  ptr->y>=0 && ptr->y<ny) {
	      tot1=ptr->data[4];
	      analyze_cluster(*ptr, &t2max, &tot3, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
	      noise=noise_map[ptr->y*nx+ptr->x];
	      if (tot1>noise && t2max>2*noise && tot3>3*noise) {
		;
	      } else
		good=0;
	    } else {
	      printf("Bad pixel number %d %d\n",ptr->x,ptr->y);
	      good=0;
	    }
	  }
	  if (good) {
	    ptr++;
	    nph_read++;
	    (*n_left)--;
	  }
	  if (nph_read >= n_clusters)
	    break;
	}
    }

    if (nph_read < n_clusters) {
        // keep on reading frames and photons until reaching n_clusters
        while (fread(&iframe, sizeof(iframe), 1, fp)) {
	  //printf("%d\n",nph_read);

	  if (fread(&nph, sizeof(nph), 1, fp)) {
	    //printf("** %d\n",nph);
	    *n_left = nph;
	    for (int iph=0; iph<nph; iph++) {
	      // read photons 1 by 1
	      fread((void *)(ptr), sizeof(Cluster), 1, fp);
	      good=1;
	      if (noise_map) {
		if (ptr->x>=0 && ptr->x<nx &&  ptr->y>=0 && ptr->y<ny) {
		  tot1=ptr->data[4];
		  analyze_cluster(*ptr, &t2max, &tot3, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
		  noise=noise_map[ptr->y*nx+ptr->x];
		  if (tot1>noise && t2max>2*noise && tot3>3*noise) {
		    ;
		  } else
		    good=0;
		} else{
		  printf("Bad pixel number %d %d\n",ptr->x,ptr->y);
		  good=0;
		}
	      }
	      if (good) {
		ptr++;
		nph_read++;
		(*n_left)--;
	      }
	      if (nph_read >= n_clusters)
		break;

	    }
	  }
	  
	  if (nph_read >= n_clusters)
	    break;
	}
    }
    //printf("%d\n",nph_read);
    assert(nph_read <= n_clusters); // sanity check in debug mode
    return nph_read;
}







int analyze_clusters(int64_t n_clusters, Cluster *cin, ClusterAnalysis *cout) {

    int32_t tot2[4], t2max;
    char quad;
    int32_t val, tot3;
    for (int ic = 0; ic < n_clusters; ic++) {
        

	analyze_cluster(*(cin+ic), &t2max, &tot3, &quad, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

        (cout + ic)->c = quad;
        (cout + ic)->tot2 = t2max;
        (cout + ic)->tot3 = tot3;
        // printf("%d %d %d %d %d %d\n",ic,(cin+ic)->x, (cin+ic)->y,
        // (cout+ic)->c, (cout+ic)->tot2, (cout+ic)->tot3);
    }
    return n_clusters;
}



int analyze_cluster(Cluster cin, int32_t *t2, int32_t *t3, char *quad, double *eta2x, double *eta2y, double *eta3x, double *eta3y, double *eta2Lx, double *eta2Ly, double *eta3Xx, double *eta3Xy) {


  int ok=1;

    int32_t tot2[4], t2max;
    char c;
    int32_t val, tot3;
    
    tot3 = 0;
    for (int i = 0; i < 4; i++)
      tot2[i] = 0;
    // t2max=0;
    for (int ix = 0; ix < 3; ix++) {
      for (int iy = 0; iy < 3; iy++) {
	val = cin.data[iy * 3 + ix];
	tot3 += val;
	if (ix <= 1 && iy <= 1)
	  tot2[0] += val;
	if (ix >= 1 && iy <= 1)
	  tot2[1] += val;
	if (ix <= 1 && iy >= 1)
	  tot2[2] += val;
	if (ix >= 1 && iy >= 1)
	  tot2[3] += val;
      }
    }



    if (t2 || quad) {
      t2max = tot2[0];
      c = cBottomLeft;
      for (int i = 1; i < 4; i++) {
	if (tot2[i] > t2max) {
	  t2max = tot2[i];
	  c = i;
	}
      }
    }
    if (quad)
      *quad = c;
    if (t2)
      *t2 = t2max;
    if (t3)
      *t3 = tot3;





    return ok;


}  
